PROGRAM GetPop3;

{&use32+}
{$h+}

uses
  uTwmSockDef,
  uTwmSockFunc,
  uTWMSocket,
  uTwmPop3Client,
  dos,
  sysutils,
  vputils,
{$ifdef os2}
  OS2Def,
  OS2Base,
{$else}
  windows,
  os2comp,
{$endif}
  crt,
  uTwmDos,
  uTwmStringS;

{$L IBMTCPIP.LIB}

const
  cHostname: AnsiString='';
  cUsername: AnsiString='';
  cPassword: AnsiString='';
  cDestDir: AnsiString='';
  cReadLogFile: AnsiString='';
  cWriteLogFile: AnsiString='';

type
  tFileMessage=class(tPop3Message)
    fHandle: ULONG;
    fFilename: AnsiString;
    constructor Create;
    destructor Destroy; override;
    procedure AddLine(_Line: AnsiString); override;
  end;

type
  tMyPop3Client=class(tPop3Client)
    procedure WriteLine(_Line: string);
  end;

procedure tMyPop3Client.WriteLine(_Line: string);
begin
  WriteLn(_Line);
end;

var
  Pop3Client: tMyPop3Client;

function FileCreateNew(FileName: PChar; var Handle: Longint): ApiRet;
  var
{$ifdef os2}
    rc : integer;
    Action : integer;
{$else}
    SA: TSecurityAttributes;
{$endif}
  begin
{$ifdef os2}
  rc:=DosOpen(FileName, Handle, Action, 0, 0,
              open_action_create_if_new OR open_action_fail_if_exists,
              open_share_DenyWrite OR open_access_WriteOnly, NIL);
  if (rc=0) and (Action=file_created) then
    Result:=0
  else
    Result:=rc;
{$else}
  SA.nLength := SizeOf(SA);
  SA.lpSecurityDescriptor := nil;
  SA.bInheritHandle := True;
  Handle := CreateFile(FileName, generic_Write, 0, @SA, create_New,
                       file_attribute_Normal, 0);
  if Handle <> invalid_Handle_Value then
    Result := 0
  else
    Result := GetLastError;
{$endif}
end;

function OpenUniqueFile(const _prefix, _extension: AnsiString; var _FileName: AnsiString): integer;
  var
    i : integer;
    rc: ApiRet;
  begin
  i:=0;
  repeat
    _FileName:=_Prefix+Long2Dec4(i)+_Extension+#0;
    Inc(i);
    rc:=FileCreateNew(pChar(_Filename), Result);
  until (rc=0);
end;

constructor tFileMessage.Create;
  begin
  fHandle:=OpenUniqueFile(cDestDir+'GPOP','.MSG', fFilename);
end;

destructor tFileMessage.Destroy;
  begin
  FileClose(fHandle);
end;

procedure tFileMessage.AddLine(_Line: AnsiString);
  const
    CrLf: AnsiString = #13#10;
  var
    BytesWritten: ULONG;
  begin
  if _Line<>'' then begin
    BytesWritten:=FileWrite(fHandle, pChar(_Line)^, Length(_Line));
    if Length(_Line)<>BytesWritten then
      Raise Exception.Create('Error writing file '+fFilename+'.');
  end;
  BytesWritten:=FileWrite(fHandle, pChar(CrLf)^, 2);
  if 2 <> BytesWritten then
    Raise Exception.Create('Error writing file '+fFilename+'.');
end;

procedure ReadMsg(_MsgNo: integer);
  var
    Msg: tFileMessage;
  begin
  Msg:=tFileMessage.Create;
  try
    Pop3Client.GetMsg(_MsgNo, Msg);
  finally
    Msg.Destroy;
  end;
end;

procedure ReadMsgs(_Cnt: integer);
  var
    i: integer;
  begin
  for i:=1 to _Cnt do begin
    writeln(i/_cnt*100:2:2, '%');
    try
      ReadMsg(i);
      Pop3Client.DeleteMsg(i);
    except
      on e: exception do begin
        WriteLn;
        WriteLn('an exception ', e.ClassName, ' has occured. Message:');
        WriteLn(e.Message);
      end;
    end;
  end;
end;

procedure GetTheMail;
  begin
  Pop3Client:=tMyPop3Client.CreateEx(cHostname, cUsername, cPassword);
  Pop3Client.OnLogLine:=Pop3Client.WriteLine;
  try
    Pop3Client.Open;
    try
      ReadMsgs(Pop3Client.MsgsAvail);
    finally
      Pop3Client.Close;
    end;
  finally
    Pop3Client.Free;
    Pop3Client:=nil;
  end;
end;

procedure Usage;
  begin
  WriteLn('usage:');
  WriteLn;
  WriteLn('GetPop3 [options] <pop3 server> <username> <password> [<destdir>]');
  WriteLn('where');
  WriteLn('<pop3 server> = name of the pop3 server');
  WriteLn('<username> and <password> = user name and password to retrieve mail for');
  WriteLn('<destdir> = directory to save MSG files in, default: current directory');
  WriteLn('options:');
  WriteLn('  -R<readlogfile>  = write all incoming data to <readlogfile>');
  WriteLn('  -W<writelogfile> = write all outgoing data to <writelogfile>');
  Halt;
end;

procedure GetParams;
  var
    i: integer;
  begin
  i:=1;
  while (i<ParamCount) and (LeftStr(ParamStr(i), 1)='-') do begin
    case UpCase(NthCharOf(ParamStr(i), 2)) of
      'R': cReadLogFile:=Trim(TailStr(ParamStr(i),3));
      'W': cWriteLogFile:=Trim(TailStr(ParamStr(i),3));
    end;
    Inc(i);
  end;
  if (ParamCount<i+2) or (ParamCount>i+3) then
    Usage;
  cHostname:=ParamStr(i);
  cUsername:=ParamStr(i+1);
  cPassword:=ParamStr(i+2);
  if ParamCount=i+3 then
    cDestDir:=AddBackslash(ParamStr(i+3));
end;

procedure Main;
  begin
  popuperrors:=false;
  try
    WriteLn('GetPop3 v1.01.000 (c) 1998 by Thomas W. Mueller');
    WriteLn;
    WriteLn('Download emails from a Pop3 server to files.');
    WriteLn('Written with Virtual Pascal v2.0');
    WriteLn;
    GetParams;
    GetTheMail;
  except
    ON e: Exception do begin
      WriteLn;
      WriteLn('an exception ', e.ClassName, ' has occured. Message:');
      WriteLn(e.Message);
    end;
  end;
end;

begin
  Main;
end.

