unit uTwmSockFunc;

(*
 * Copyright (c) 1982, 1985, 1986 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that this notice is preserved and that due credit is given
 * to the University of California at Berkeley. The name of the University
 * may not be used to endorse or promote products derived from this
 * software without specific prior written permission. This software
 * is provided ``as is'' without express or implied warranty.
 *
 *      @(#)socket.h    7.2 (Berkeley) 12/30/87
 *)

(* VP conversion by Thomas Mueller *)

interface

uses
{$ifdef OS2}
  OS2Def,
  IBMSO32,
  IBMTCP32,
{$else}
  windows,
  uW32Sock,
  Os2Comp,  
{$endif os2}
  Sysutils,
  uTwmSockDef;

  function SockErrorNo: ApiRet;
  procedure SockWriteError(_prefix: ShortString);
  function SockGetErrStr(_ErrNo: integer): ShortString;
  procedure SockRaiseError(const _prefix: String; _ErrNo: integer);
  procedure SockRaiseLastError(const _prefix: String);

  function SockAccept(_SockDesc: tSockDesc; _SockAddr: pSockAddr;
                      var _SockAddrLen: ULong): tSockDesc;
  function SockBind(_SockDesc: tSockDesc; const _SockAddr: tSockAddr): ApiRet;
  function SockCancel(_SockDesc: tSockDesc): ApiRet;
  function SockConnect(_SockDesc: tSockDesc; const _SockAddr: tSockAddr): ApiRet;
  function SockShutdown(_SockDesc: tSockDesc; _how: ULong): ApiRet;
{$ifdef os2}
  function SockGetHostID: ULong;
{$endif}
  function SockGetSockAddr(_SockDesc: tSockDesc; var _SockAddr: tSockAddr): Longint;
  function SockGetSockOpt(_SockDesc: tSockDesc; _Level, _OptName: ULong;
                          _OptVal: PChar; var _OptLen: ULong): ApiRet;
  function SockSetSockOpt(_SockDesc: tSockDesc; _Level: uLong; _OptName: Ulong;
                          _OptVal: pChar; _OptLen: uLong ): ApiRet;
  function SockSetBlockingIO(_SockDesc: tSockDesc; _BlockingIO: boolean): ApiRet;
  function SockListen(_SockDesc: tSockDesc; _SockQueue:  ULong): ApiRet;
  function SockRecv(_SockDesc: tSockDesc; _SockBuffer: pointer;
                    _SockBufLen: ULong; _SockFlags:  ULong): ApiRet;
  function SockSend(_SockDesc: tSockDesc; _SockBuffer: pointer;
                    _SockBufLen: ULong; _SockFlags:  ULong ): ApiRet;
  function SockSocket(_SockFamily: word; _SockType: word;
                      _SockProtocol: word ): tSockDesc;
  function SockClose(_SockDesc: tSockDesc): ApiRet;
  function SockErrNo: ApiRet;
  function SockInit: ApiRet;

  function SockGetHostAddrByName(_HostName: ShortString): ULong;
  function SockGetHostNameByAddr(_HostAddr: pIn_Addr): ShortString;
  function SockGetHostname: ShortString;

  function SockGetServByName(_Name, _Proto: ShortString): pServEnt;
  function SockGetServPortByName(_Name, _Proto: ShortString): SmallInt;

  function SockHtonl(_Input: LongInt): longint;
  function SockHtons(_Input: SmallInt): SmallInt;

  function SockNtohl(_Input: LongInt): longint;
  function SockNtohs(_Input: SmallInt): SmallInt;
{$ifdef os2}
  function SockDataAvail(_SockDesc: tSockDesc): ulong;
{$endif}
  function SockSelect(_SockDesc: tSockDesc): ApiRet;

  function SockInetAddr(_s: ShortString):tIn_Addr;

{$R uTwmSockFunc.res}

implementation

  const
    Version    = '1.00';
    UseString:  ShortString = '@(#)socket interface unit for IBM TCP/IP and WinSock'#0;
    CopyRight1: ShortString = '@(#)socket Version '+Version+' - 26.08.1998'#0;
    CopyRight2: ShortString = '@(#}(C) Thomas Mueller 1998'#0;
    CopyRight3: ShortString = '@(#)(C) Chr.Hohmann BfS ST2.2 1996'#0;

  function SockErrorNo: ApiRet;
    begin
    {$ifdef os2}
      Result:=IBM_sock_errno;
    {$else}
      Result:=MS_GetLastError;
    {$endif}
  end;

  function SockGetErrStr(_ErrNo: integer): ShortString;
    begin
    Result:=LoadStr(_ErrNo);
  end;

  procedure SockRaiseError(const _prefix: String; _ErrNo: integer);
    begin
    raise eSocketErr.CreateFmt('%s: %s (%d)', [_prefix, SockGetErrStr(_ErrNo), _ErrNo]);
  end;

  procedure SockRaiseLastError(const _prefix: String);
    begin
    SockRaiseError(_Prefix, SockErrorNo);
  end;

  procedure SockWriteError(_prefix: ShortString);
    {$ifndef os2}
    var
      s: string;
      err: integer;
    {$endif}
    begin
  {$ifdef os2}
    _prefix := _prefix+#0;
    IBM_psock_errno(@_prefix[1]);
  {$else}
    Err:=SockErrorNo;
    s:=SockGetErrStr(Err);
    if s='' then
      s:=Format('<no errormessage available for error #%d>', [Err]);
    if _Prefix<>'' then
      s:=_Prefix+': '+s;
    FileWrite(SysFileStdErr, s[1], Length(s));
  {$endif}
  end;

  function SockGetServByName(_Name, _Proto: ShortString): pServEnt;
    begin
    _Name:=_Name + #0;
    _Proto:=_Proto + #0;
    Result:=getservbyname(@_Name[1], @_Proto[1]);
  END;

  function SockGetServPortByName(_Name, _Proto: ShortString): SmallInt;
    VAR
      ServEnt: pServEnt;
    begin
    ServEnt := SockGetServByName( _Name, _Proto );
    if Assigned(ServEnt) then
      Result := ServEnt^.s_Port
    else
      result:=-1;
  END;

  function SockHtonl( _Input: longint): longint;
    begin
    Result:=htonl(_Input);
  END;

  function SockHtons( _Input: SmallInt ): SmallInt;
    begin
    Result := htons( _input );
  end;

  function SockNtohl( _Input: longint): longint;
    begin
    Result:=ntohl(_Input);
  END;

  function SockNtohs( _Input: SmallInt ): SmallInt;
    begin
    Result := ntohs( _input );
  end;

  function SockAccept(_SockDesc:  tSockDesc;
                      _SockAddr:    pSockAddr;
                      VAR _SockAddrLen: ULong): ApiRet;
    begin
    Result:=accept(_SockDesc, _SockAddr, @_SockAddrLen);
  end;

  function SockBind(_SockDesc:  tSockDesc;
                    const _SockAddr: tSockAddr ): ApiRet;
    begin
    SockBind:=bind(_SockDesc,
                   @_SockAddr,
                   SockAddr_Len);
  end;

  function SockConnect(_SockDesc: tSockDesc;
                       const _SockAddr: tSockAddr): ApiRet;
    begin
    SockConnect := connect(_SockDesc,
                           @_SockAddr,
                           SockAddr_Len);
  end;
  (****************************************************************************)
  function SockCancel(_SockDesc: tSockDesc): ApiRet;
    begin
  {$ifdef os2}
    result:=so_cancel(_SockDesc);
  {$else}
    Result:=MS_CancelBlockingCall;
  {$endif}
  end;
  (****************************************************************************)
  function SockShutdown(_SockDesc:  tSockDesc;
                        _how: ULong): ApiRet;
    begin
    SockShutdown:=shutdown(_SockDesc, _how);
  end;

  (****************************************************************************)
{$ifdef os2}
  function SockGetHostID: ULong;
    begin
    SockGetHostID := SockHtonl(gethostid);
  end;
{$endif}
  (****************************************************************************)
  function SockGetSockAddr(_SockDesc: tSockDesc; var _SockAddr: tSockAddr): Longint;
    var
      SLength: longint;
    begin
    fillchar(_SockAddr,sizeof(_SockAddr),#0);
    SLength:=SizeOf(_SockAddr);
    result:=getsockname(_SockDesc, @_SockAddr, SLength);
  end;
  (****************************************************************************)
  function SockSetBlockingIO(_SockDesc: tSockDesc;
                             _BlockingIO: boolean): ApiRet;
  {$ifdef os2}
    var
      Arg: ULong;
  {$endif}
    begin
  {$ifdef os2}
    if _BlockingIO then
      Arg:= 0
    else
      Arg:= 1;
    Result:=ioctl(_SockDesc, FIONBIO, @Arg, SizeOf(Arg));
  {$else}
    Result:=-1;
  {$endif}
  end;
  (****************************************************************************)
  function SockGetSockOpt(_SockDesc: tSockDesc; _Level, _OptName: ULong;
                          _OptVal: PChar; var _OptLen: ULong): ApiRet;
    begin
    Result:=GetSockOpt(_SockDesc, _Level, _OptName, _OptVal, _OptLen);
  end;
  (****************************************************************************)
  function SockSetSockOpt(_SockDesc: tSockDesc; _Level: uLong; _OptName: Ulong;
                          _OptVal: pChar; _OptLen: uLong ): ApiRet;
    begin
    Result:=SetSockOpt(_SockDesc, _Level, _OptName, _OptVal, _OptLen);
  end;

{$ifdef os2}
  (****************************************************************************)
  function SockDataAvail(_SockDesc: tSockDesc): ULONG;
    begin
    if ioctl(_SockDesc, FIONREAD, @Result, SizeOf( Result ) ) = -1 then
      Result:=$FFFFFFFF;
  end;
{$endif}

  (****************************************************************************)
  function SockListen(_SockDesc: tSockDesc;
                      _SockQueue:  ULong): ApiRet;
    begin
    SockListen := listen(_SockDesc, _SockQueue);
  end;
  (****************************************************************************)
  function SockSelect(_SockDesc: tSockDesc ): ApiRet;
    var
    {$ifdef os2}
      SockCopy: ULONG;
    {$else}
      SockArr: TFDSet;
      Timeout: TTimeVal;
    {$endif}
    begin
  {$ifdef os2}
    SockCopy:=_SockDesc;
    Result:=IBM_select(@SockCopy, 1, 0, 0, 0);
  {$else}
    SockArr.fd_count:=1;
    SockArr.fd_array[0]:=_SockDesc;
    Timeout.tv_sec:=0;
    Timeout.tv_usec:=0;
    Result:=MS_select(0, @SockArr, nil, nil, @Timeout);
  {$endif}
  end;
  (****************************************************************************)
  function SockRecv(_SockDesc: tSockDesc;
                    _SockBuffer: pointer;
                    _SockBufLen: ULong;
                    _SockFlags:  ULong): ApiRet;
    begin
    SockRecv := recv(_SockDesc,
                     _SockBuffer,
                     _SockBufLen,
                     _SockFlags);
  end;
  (****************************************************************************)
  function SockSend(_SockDesc: tSockDesc;
                    _SockBuffer: pointer;
                    _SockBufLen: ULong;
                    _SockFlags:  ULong): ApiRet;
    begin
    SockSend := send(_SockDesc,
                         _SockBuffer,
                         _SockBufLen,
                         _SockFlags);
  end;
  (****************************************************************************)
  function SockSocket(_SockFamily:   word;
                      _SockType:     word;
                      _SockProtocol: word): tSockDesc;
    begin
    SockSocket := socket(_SockFamily, _SockType, _SockProtocol);
  end;
  (****************************************************************************)
  function SockClose(_SockDesc: tSockDesc): ApiRet;
    begin
  {$ifdef os2}
    Result:=IBM_soclose(_SockDesc);
  {$else}
    Result:=MS_closesocket(_SockDesc);
  {$endif}
  end;
  (****************************************************************************)
  function SockErrNo: ApiRet;
    begin
  {$ifdef os2}
    SockErrNo := IBM_Sock_ErrNo;
  {$else}
    Result:=MS_getlasterror;
  {$endif}
  end;
  (****************************************************************************)
  function SockInit: ApiRet;
    var
      Data: TWSAData;
    begin
  {$ifdef os2}
    SockInit := Sock_Init;
  {$else}
    SockInit:=MS_Startup($0101, @Data);
  {$endif}
  end;
  (****************************************************************************)
  function SockGetHostByName(Hostname: ShortString): pointer;
    begin
    HostName:=HostName+#0;
    Result:=gethostbyname(@HostName[1]);
  end;
  (****************************************************************************)
  function SockGetHostAddrByName(_HostName: ShortString): ULong;
  var
    rc:     pHostEnt;
    InAddr: tIn_Addr;
    begin
    rc:=SockGetHostByName(_Hostname);
    if Assigned(rc) then begin
      InAddr:=rc^.H_Addr_List^^;
      Result:=InAddr.IPAddr;
    end else
      Result:=$FFFFFFFF;
  end;
  (****************************************************************************)
  function SockGetHostByAddr(HostAddr:     pIn_Addr;
                             HostAddrLen:  ULong;
                             HostAddrType: ULong): pointer;
    begin
    SockGetHostByAddr := gethostbyaddr(HostAddr,
                                           HostAddrLen,
                                           HostAddrType);
  end;
  (****************************************************************************)
  function SockGetHostNameByAddr(_HostAddr: pIn_Addr): ShortString;
  var
    I:     integer;
    rc:    pHostEnt;
    HName: ShortString;
    begin
    rc := SockGetHostByAddr(_HostAddr,
                            In_Addr_Len,
                            AF_INET);
    if (ULong(rc) <> 0) then begin
      HName := '';
      I     := 0;
      while rc^.H_Name^[I] <> #0 do begin
        HName := HName+rc^.H_Name^[I];
        inc(I);
      end;
    end else HName := 'hostname not found';
    SockGetHostNameByAddr := HName;
  end;
  (****************************************************************************)
  function SockGetHostname: ShortString;
  var
    I,
    SResult:  longint;
    HostName: ShortString;
    begin
    fillchar(HostName,sizeof(HostName),#0);
    SResult:=gethostname(@HostName[1], sizeof(HostName));
    I := 1;
    while (I < sizeof(HostName)) and (Hostname[I] <> #0) do inc(I);
    if (I > 1) then
      SetLength(HostName, I)
    else
      HostName    := 'amnesiac';
    SockGetHostname := HostName;
  end;
  (****************************************************************************)
  function SockInetAddr(_s: ShortString): tIn_Addr;
    begin
    _s := _s+#0;
    result.IPAddr := inet_addr( @_s[1] );
  END;

initialization
  SockInit;
end.

